#!/bin/sh
PACK_ONLY=0

if [ "X$1" = "X--pack-only" ]; then
  PACK_ONLY=1
fi

. ./build-env.sh

export ARCH=arm64
export KBUILD_DEBARCH=arm64
export K_IMAGE=Image
export KBUILD_IMAGE=${K_IMAGE}
if [ "X${BSP_ARCH}" = "Xarmhf" ]; then
  export ARCH=arm
  export CROSS_COMPILE=arm-linux-gnueabihf-
  export KBUILD_DEBARCH=armhf

elif [ "X${BSP_ARCH}" = "Xarm64" ]; then
  export ARCH=arm64
  export CROSS_COMPILE=aarch64-linux-gnu-
  export KBUILD_DEBARCH=arm64

elif [ "X${BSP_ARCH}" = "Xriscv64" ]; then
  export ARCH=riscv
  export CROSS_COMPILE=riscv64-linux-gnu-
  export KBUILD_DEBARCH=riscv64

  t=`pwd`/prebuilt/gcc/linux-x86/riscv/toolchain-thead-glibc/riscv64-glibc-gcc-thead_20200702/bin/riscv64-unknown-linux-gnu-
  if [ -e ${t}gcc ]; then
    export CROSS_COMPILE=$t
  fi

elif [ "X${BSP_ARCH}" = "Xamd64" ]; then
  export ARCH=x86
  export CROSS_COMPILE=x86_64-linux-gnu-
  export KBUILD_DEBARCH=amd64
  export KBUILD_IMAGE=bzImage
  export K_IMAGE=bzImage
  export KBUILD_IMAGE=arch/${ARCH}/boot/${K_IMAGE}

elif [ "X${BSP_ARCH}" = "Xi686" ]; then
  export ARCH=x86
  export CROSS_COMPILE=i686-linux-gnu-
  export KBUILD_DEBARCH=i386
  export KBUILD_IMAGE=bzImage
  export K_IMAGE=bzImage
  export KBUILD_IMAGE=arch/${ARCH}/boot/${K_IMAGE}

elif [ "X${BSP_ARCH}" = "Xpowerpc" ]; then
  export ARCH=powerpc
  export CROSS_COMPILE=powerpc-linux-gnu-
  export KBUILD_DEBARCH=powerpc
fi

if [ android_bootimg_tools/mkbootimg.c -a ! -e android_bootimg_tools/mkbootimg ]; then
  cd android_bootimg_tools/
  make
  cd ..
fi

cd linux/

if [ ! -e .builddate ]; then
  LANG=C date -u > .builddate
fi

VERSION=`cat Makefile | grep -E '^VERSION.=' | cut -d "=" -f 2- | tr -d " \t\n\r"`
PATCHLEVEL=`cat Makefile | grep PATCHLEVEL.= | cut -d "=" -f 2- | tr -d " \t\n\r"`
SUBVERSION=`cat Makefile | grep SUBLEVEL.= | cut -d "=" -f 2- | tr -d " \t\n\r"`
BUILDVERSION=1
BUILDTIMESTAMP=`cat .builddate`

export LOCALVERSION="+${BSP_GROUP}"
export KBUILD_BUILD_TIMESTAMP=${BUILDTIMESTAMP}
export KBUILD_BUILD_VERSION=${BUILDVERSION}
export KDEB_PKGVERSION="${VERSION}.${PATCHLEVEL}.${SUBVERSION}-20211210"
export KZIP_PKGVERSION="${KDEB_PKGVERSION}-${BSP_MODEL}"
export KDEB_PKGVERSION="${KDEB_PKGVERSION}-${BSP_GROUP}"

echo "Building ${KZIP_PKGVERSION} (${KDEB_PKGVERSION})..."

kmake() {
  echo ""
  echo "make $*"

  #KBUILD_IMAGE="Image"
  #KBUILD_IMAGE=${KBUILD_IMAGE}
    make $* KDEB_PKGVERSION="${KDEB_PKGVERSION}" \
  LOCALVERSION="${LOCALVERSION}" ARCH=${ARCH} CROSS_COMPILE=${CROSS_COMPILE} \
  KBUILD_BUILD_TIMESTAMP="${KBUILD_BUILD_TIMESTAMP}" KBUILD_BUILD_VERSION=${KBUILD_BUILD_VERSION} \
  KBUILD_DEBARCH=${KBUILD_DEBARCH} \
  DEBFULLNAME="SCP" \
  DEBEMAIL="scsupport@detc.local"
}

if [ $PACK_ONLY = 0 ] ; then
  kmake -j4 ${K_IMAGE} modules
  if [ "X${BSP_DTS}" != "Xnone" ]; then
    kmake dtbs
  fi

  kmake headers_check 
  kmake scripts
fi

dtm=none
dtb=none.dtb

for m in '/*'${BSP_DTS} ${BSP_DTS} '/*odroid' '/*rockpro64' '/*rock64' 'rock2' ; do
  if [ ! -e $dtb ]; then
    dtm=$m
    for f in arch/$ARCH/boot/dts/*${dtm}*.dtb; do
      if [ -e $f ]; then
        if ! echo $f | grep -q android ; then
          dtb=$f
        fi
      fi
    done
  fi
done

if [ -e scripts/package/.builddeb.template ]; then
  cp -p scripts/package/.builddeb.template scripts/package/builddeb
else
  cp -p scripts/package/builddeb scripts/package/.builddeb.template
fi

#if [ -e arch/${ARCH}/boot/dts ]; then
#  patch -p1 < ../patches/linux-4.4.y-rk-debpkg-add-dts.patch
#fi

if [ -e $dtb ]; then
  if grep -q -E '/\*odroid' scripts/package/builddeb ; then
    sed -i "s|/\*odroid|${dtm}|g" scripts/package/builddeb
  fi
fi

if [ $PACK_ONLY = 0 ] ; then
  #kmake -j1 deb-pkg 
  kmake -j1 bindeb-pkg
fi

cd ..

if [ $PACK_ONLY = 0 ] ; then
  bash -e build-vdec-fw.sh
fi

mkdir -p dbg
mv linux-*${BSP_GROUP}*-dbg*_${KDEB_PKGVERSION}_*.deb dbg/ || true

if [ -e .install-dtb.sh.template ]; then
  cp -p .install-dtb.sh.template install-dtb.sh
else
  cp -p install-dtb.sh .install-dtb.sh.template
fi
if [ -e .install-image.sh.template ]; then
  cp -p .install-image.sh.template install-image.sh
else
  cp -p install-image.sh .install-image.sh.template
fi
if [ -e .install-linux.sh.template ]; then
  cp -p .install-linux.sh.template install-linux.sh
else
  cp -p install-linux.sh .install-linux.sh.template
fi

if [ -e linux/$dtb ]; then
  echo "Using $dtb"
  dts=`basename $dtb`
  dts=${dts/.dtb/}
  dtk=`echo ${dts} | sed "s/-linux$//g"`
  dtl=`echo ${dtk} | sed "s/^meson-g..b/meson64/g" | sed "s/odroid-/odroid/g" | tr - _`

  if grep -q meson-gxbb-odroidc2 install-dtb.sh ; then
    sed -i "s|meson64_odroidc2|${dtl}|g" install-dtb.sh
    sed -i "s|meson-gxbb-odroidc2|${dtk}|g" install-dtb.sh

  elif grep -q rk3399-odroidn1-linux install-dtb.sh ; then
    sed -i "s|rk3399-odroidn1-linux|${dts}|g" install-dtb.sh
  fi

  if grep -q -E '\*sbc_|-sbc\*_' install-linux.sh ; then
    sed -i "s|-sbc\*_|-${BSP_GROUP}*_|g" install-linux.sh
    sed -i "s|\*sbc_|\*${BSP_GROUP}_|g" install-linux.sh
    sed -i "s|_arm64\.deb|_${KBUILD_DEBARCH}.deb|g" install-linux.sh
  fi

  if grep -q -E '\*c2\*' install-linux.sh ; then
    sed -i "s|\*c2\*|${dts}|g" install-linux.sh
    sed -i "s|-c2\*_|-${BSP_MODEL}*_|g" install-linux.sh
    sed -i "s|\*c2_|\*${BSP_MODEL}_|g" install-linux.sh
    sed -i "s|_arm64\.deb|_${KBUILD_DEBARCH}.deb|g" install-linux.sh

  elif grep -q -E '\*n1\*' install-linux.sh ; then
    sed -i "s|\*n1\*|${dts}|g" install-linux.sh
    sed -i "s|-n1\*_|-${BSP_MODEL}*_|g" install-linux.sh
    sed -i "s|_arm64\.deb|_${KBUILD_DEBARCH}.deb|g" install-linux.sh
  fi

  if [ ${dts:0:5} = sun7i ]; then
    echo "family ${dts:0:5}"
    sed -i 's|$kdtb /boot/${b}|`dirname ${kdtb}`/'${dts:0:16}'*.dtb /boot/|g' install-linux.sh
    sed -i "s|/${dts}.dtb |/${dts:0:16}*.dtb |g" install-linux.sh
    sed -i 's|if grep -q ${BOOTPART} /etc/fstab ;|if false ;|g' install-linux.sh
    sed -i "s|/${dts}.dtb /|/${dts:0:16}*.dtb /|g" install-dtb.sh
    sed -i "s|/media/boot/|/media/boot/dtb/|g" install-dtb.sh
    sed -i "s|cp .*boot/dtb/dtb|true|g" install-dtb.sh
    sed -i "s|/Image|/zImage-next|g" install-image.sh
  fi
else
  if grep -q -E '\*c2\*' install-linux.sh ; then
    sed -i "s|-c2\*_|-${BSP_MODEL}*_|g" install-linux.sh
    sed -i "s|\*c2_|\*${BSP_MODEL}_|g" install-linux.sh
    sed -i "s|_arm64\.deb|_${KBUILD_DEBARCH}.deb|g" install-linux.sh

  elif grep -q -E '\*n1\*' install-linux.sh ; then
    sed -i "s|-n1\*_|-${BSP_MODEL}*_|g" install-linux.sh
    sed -i "s|_arm64\.deb|_${KBUILD_DEBARCH}.deb|g" install-linux.sh
  fi
fi

ai=linux/arch/${ARCH}/boot/Image
#an=d1-nezha
an=`echo ${BSP_DTS} | cut -d '-' -f 2-`

ak=40200000
ad=
ar=41a00000
if [ ${dts:0:6} = rk3399 ]; then
  ak=02000000
  ad=01f00000
  ar=04000000

elif [ ${dts:0:10} = meson-gxbb ]; then
  ak=11000000
  ad=01000000
  ar=13000000

elif [ ${dts:0:5} = sun7i ]; then
  ak=46000000
  ad=49000000
  ar=47000000
fi

ab=`printf "%.8X\n" $(( 0x${ak} - 0x00008000 ))`
ao=`printf "%.8X\n" $(( 0x${ar} - 0x${ab} ))`
ap=2048

if [ -e android_bootimg_tools/mkbootimg -a -e ${ai} ]; then
  echo "board: ${an}, kerneladdr: ${ak}, base: ${ab} + ramdisk_offset: ${ao} = ramdiskaddr: ${ar}, pagesize: ${ap}"
  if [ ! -e ramdisk.gz ]; then
    echo "ramdisk.img" > ramdisk.gz
  fi
  if grep -q ramdiskaddr android_bootimg_tools/mkbootimg ; then
    # 41200000
    ./android_bootimg_tools/mkbootimg --kernel ${ai} --ramdisk ramdisk.gz --board ${an} --base ${ab} --ramdiskaddr ${ar} --pagesize ${ap} -o bootimg.bin
  else
    
    # 01008000
    ./android_bootimg_tools/mkbootimg --kernel ${ai} --ramdisk ramdisk.gz --board ${an} --base ${ab} --ramdisk_offset ${ao} --pagesize ${ap} -o bootimg.bin
  fi

else
  rm -f bootimg.bin
fi

echo linux-image-${KZIP_PKGVERSION}-${KBUILD_DEBARCH}.zip
zip linux-image-${KZIP_PKGVERSION}-${KBUILD_DEBARCH}.zip \
  bootimg.bin \
  install-linux.sh install-dtb.sh install-image.sh install-uinitrd.sh \
  linux-*_${KDEB_PKGVERSION}_*.deb meson-vdec-firmware_*_${KBUILD_DEBARCH}.deb

